from .entries.json_easy import JsonEasyEntry
from .__init__ import BasicCheckUtil
import json

class JsonUtil:
    '''
    JsonEasyEntry 简单操作工具类（建议使用）
    使用前建议先调用JsonUtil.print方法查看处理后的json串，便于快捷提取所需数据
    '''
    @staticmethod
    def loads(data: str or dict or list or tuple):
        '''
        装载json数据，获取JsonEeasyEntry对象
        :param data: json结构的字符串或dict对象或列表
        :return: JsonEeasyEntry
        '''
        return JsonEasyEntry(data)

    @staticmethod
    def get_json(obj: JsonEasyEntry):
        '''
        获取json对象
        :param obj: JsonEasyEntry 对象，通过JsonUtil.loads获取
        :return: dict 或 list
        '''
        JsonUtil.__check(obj)
        return obj.get_original_json()

    @staticmethod
    def get_json_str(obj: JsonEasyEntry):
        '''
        获取json字符串
        :param obj: JsonEasyEntry 对象，通过JsonUtil.loads获取
        :return: str
        '''
        JsonUtil.__check(obj)
        return obj.get_original_json_text()

    @staticmethod
    def get(obj: JsonEasyEntry, key: str, index: int or list=None, level: int=None):
        '''
        查找某个key出现的结果。
        当索引index为None时查找所有结果，返回list对象。
        当索引index为list对象时，返回所有索引index对应位置的值，list对象。
        当索引index为某个具体的数值时，返回单个索引index位置的值。
        :param obj: JsonEasyEntry 对象，通过JsonUtil.loads获取
        :param key: 查找的key
        :param:index: 索引
        :param level: 层级，第一级从0开始，查找指定层级，为None时不限制层级。（当存在不同层级出现相同key时可以使用该字段区分）
        例如： {"he":{"he": "sh"}} 时，不使用level而仅使用key时，将会得到[{"he":"sh"},{"sh"}]结果，具体按照业务场景使用
        :return:list 或 tuper
        '''
        JsonUtil.__check(obj)
        if index == None:
            return obj.get(key)
        elif type(index) in (int,):
            return obj.get_one(key, index)
        elif type(index) in (list,):
            result_list = []
            for one_index in index:
                result_list.append(obj.get_one(one_index))
            return result_list

    @staticmethod
    def print(obj: JsonEasyEntry):
        '''
        查看处理后的json
        :param obj:
        :return:
        '''
        obj.print()

    @staticmethod
    def to_json(s, *, encoding=None, cls=None, object_hook=None, parse_float=None,
        parse_int=None, parse_constant=None, object_pairs_hook=None, **kw):
        '''
        字符串转json, 引用自json.loads方法
        '''
        return json.loads(s, encoding=encoding, cls=cls, object_hook=object_hook, parse_float=parse_float,
                   parse_int=parse_int, parse_constant=parse_constant, object_pairs_hook=object_pairs_hook, **kw)

    @staticmethod
    def to_str(obj, *, skipkeys=False, ensure_ascii=False, check_circular=True,
        allow_nan=True, cls=None, indent=None, separators=None,
        default=None, sort_keys=False, **kw):
        '''
        json转字符串，引用自json.dumps方法
        ensure_ascii 默认改为false
        '''
        return json.dumps(obj, skipkeys=skipkeys, ensure_ascii=ensure_ascii, check_circular=check_circular,
                          allow_nan=allow_nan, cls=cls, indent=indent, separators=separators, default=default, sort_keys=False, **kw)

    @staticmethod
    def __check(obj, *args, **kwargs):
        if BasicCheckUtil.is_none(obj):
            raise RuntimeError("JsonEasyEntry对象不存在")


import os
from .__init__ import *
from .entries.file.xlsx_file_easy import XlsxFileEasyEntry

class FileUtil:
    '''
    文件操作工具类
    '''
    @staticmethod
    def file_create(path):
        '''
        创建文件，支持多级目录嵌套创建
        :param path: 文件相对路径/绝对路径
        :return:
        '''
        FileUtil.__check(path)
        # win 分隔符替换
        path = path.replace(r'\\', '/')
        with open(path) as f:
            pass
        return True

    @staticmethod
    def file_remove(path):
        '''
        删除文件
        :param path: 文件相对路径/绝对路径
        :return:
        '''
        FileUtil.__check(path)
        return os.remove(path)
    
    @staticmethod
    def file_read(file, buffering=8192, encoding='utf-8', errors=None, newline=None, closefd=True):
        '''
        文件读取,引用自内置函数open,对应open().read()方法
        调整encoding默认编码为utf-8
        '''
        if FileUtil.file_exists(file):
            with open(file, mode='r', buffering=buffering, encoding=encoding, errors=errors, newline=newline, closefd=closefd) as f:
                return f.read()

    @staticmethod
    def file_readlines(file, buffering=8192, encoding='utf-8', errors=None, newline=None, closefd=True):
        '''
        文件读取,引用自内置函数open,对应open().readlines()方法
        调整encoding默认编码为utf-8
        '''
        if FileUtil.file_exists(file):
            with open(file, mode='r', buffering=buffering, encoding=encoding, errors=errors, newline=newline,
                      closefd=closefd) as f:
                return f.readlines()

    @staticmethod
    def file_exists(path):
        '''
        检测文件是否存在
        :param file_path: 文件相对路径/绝对路径
        :return:
        '''
        FileUtil.__check(path)
        return FileUtil.__exists(path) and os.path.isfile(path)

    @staticmethod
    def dir_create(path):
        '''
        创建目录，支持多级目录嵌套创建
        :param path: 目录相对路径/绝对路径
        :return:
        '''
        FileUtil.__check(path)
        if not FileUtil.__exists(path):
            os.makedirs(path)
        return True
    
    @staticmethod
    def dir_remove(path):
        '''
        删除目录，支持非空目录删除
        :param path: 目录相对路径/绝对路径
        :return: 
        '''
        import shutil
        shutil.rmtree(path)
        return True

    @staticmethod
    def dir_exists(path):
        '''
        检测目录是否存在
        :param dir_path: 目录相对路径/绝对路径
        :return:
        '''
        FileUtil.__check(path)
        return FileUtil.__exists(path) and os.path.isdir(path)

    @staticmethod
    def dir_or_file_exists(path):
        '''
        检测路径是文件或目录
        :param path: 相对路径/绝对路径
        :return:
        '''
        FileUtil.__check(path)
        return FileUtil.__exists(path)

    @staticmethod
    def __exists(path):
        '''
        检测目录/文件是否存在
        :param path: 目录/文件 相对路径/绝对路径
        :return:
        '''
        FileUtil.__check(path)
        return os.path.exists(path)

    @staticmethod
    def __check(path, *args, **kwargs):
        '''
        路径检测
        :param path: 相对路径/绝对路径
        :return:
        '''
        if BasicCheckUtil.is_empty(path):
            raise ValueError('路径不能为空')

class XlsxFileUtil:

    @staticmethod
    def create(filepath, default_sheet=None):
        '''
        装载xlsx数据，获取XlsxFileEeasyEntry对象
        :param filepath: 文件相对路径/绝对路径
        :param default_sheet: sheet名或sheet索引，默认sheet，初始化设置默认sheet后续操作可不设置sheet。
        :return: XlsxFileEeasyEntry对象
        '''
        return XlsxFileEasyEntry(filepath, default_sheet)

    @staticmethod
    def rows(obj: XlsxFileEasyEntry, sheet=None):
        '''
        获取行数
        :param obj: XlsxFileEasyEntry对象，可通过XlsxFileUtil.create获取
        :param sheet: xlsx 查询的sheet名或sheet索引
        :return: int
        '''
        return obj.rows(sheet)

    @staticmethod
    def cols(obj: XlsxFileEasyEntry, sheet=None):
        '''
        获取列数
        :param obj: XlsxFileEasyEntry对象，可通过XlsxFileUtil.create获取
        :param sheet: xlsx 查询的sheet名或sheet索引
        :return: int
        '''
        return obj.cols(sheet)

    @staticmethod
    def find(obj: XlsxFileEasyEntry, sheet=None, start_row=None, start_col=None, end_row=None, end_col=None):
        '''
        基于位置查找xlsx文件中的内容
        支持单行查找：start_row=number, 单列查找：start_col=number，返回list
        支持指定元素查找：start_row=number1, start_col=number2，返回str
        支持范围查找：start_row=number1, start_col=number2, end_row=number3, end_col=number4, 返回list[list]
        其中参数要求：
        start_row 与 start_col 不能同时为空
        end_row 不为空时 start_row 也不能为空
        end_col 不为空时 start_col 也不能为空
        :param sheet: sheet名，值为None时使用默认sheet名
        :param start_row: 开始行数，0开始
        :param start_col: 开始列数，0开始
        :param end_row: 结束行数，1开始
        :param end_col: 结束列数，1开始
        :return: list 或 str
        '''
        if BasicCheckUtil.is_none(start_row) and BasicCheckUtil.is_none(start_col):
            raise ValueError('start_row 与 start_col 不能同时为空')
        return obj.find(sheet, start_row, start_col, end_row, end_col)

    @staticmethod
    def sheet_names(obj: XlsxFileEasyEntry):
        '''
        获取所有的sheet名
        :return:
        '''
        return obj.sheets()


from .__init__ import *
import json


class MysqlEasyConnection:
    '''
    todo 基于PyMysql模块的封装实体，SR1
    '''
    __conn = None

    def __init__(self, *args, **kwargs):
        '''
        创建连接，无返回值，内置保存连接
        :param host: Mysql Host
        :param port: Mysql Port
        :param username: Mysql Username
        :param password: Mysql Password
        :param db: Mysql DataBase Name
        :return:
        '''
        import pymysql
        self.__conn = pymysql.connect(*args, **kwargs)

    def insert_dict_auto_add_table_and_column(self, table, data, mode='into',
                                              value_list_split=',',
                                              primary_name='id', engine=None, charset=None):
        '''
        插入单条字典类型，如果表不存在，则创建表，如果表中不包含某个列，将会在最后自动新增一列
        :param table: 表
        :param data: 字典结构数据
        :param mode:模式,into overwrite ignore 等，默认 init（追加）
        :param engine: Mysql自动建表使用的引擎
        :param charset: Mysql自动建表的字符集
        :return:
        '''
        self.__check_insert(table, data)
        db_table_inf = table.split('.')
        # 连接时未设置默认数据库，则此处设置默认数据库
        if len(db_table_inf) == 2:
            self.__use_database(db_table_inf[0])
        table = db_table_inf[-1]
        if not self.exists_table(table):
            # 通过数据生成表
            self.__data_generate_table(table, data.copy(), value_list_split, primary_name, engine=engine, charset=charset)

        # 获取表的所有列名
        columns = self.__table_column_names(table)
        # 新数据key与列名集合的差集
        diff_key = data.keys() - columns
        # 差集大于 0 表示有新列需要创建
        if len(diff_key) > 0:
            diff_set = {}
            # 循环插入差集数据
            for k, v in data.items():
                if k in diff_key:
                    diff_set[k] = v
            # python 数据转python数据类型，例如 '这是一个字符串' => str, 200 => int
            diff_set = self.__dict_to_type(diff_set, value_list_split)
            # 批量生成列
            self.__dict_add_column(table, diff_set)
        return self.insert_dict(table, data, mode, value_list_split)

    def insert_dict(self, table, data, mode='into', value_list_split=','):
        '''
        插入单条字典结构数据
        :param table: 表名
        :param data: 字典结构数据
        :param mode: 插入模式，into,overwrite,ignore 等，默认into（追加）
        :param value_list_split: 当data某条数据的某个值为tupper或者list，通过此字段做分隔符转换为字符串
        :return:
        '''
        # 基本检测
        self.__check_insert(table, data)
        self.__check_exists_table(table)
        # 获取连接对象
        conn = self.__conn
        # 获取所有的key
        keys = ",".join(map(lambda x: f'`{x}`', data.keys()))
        # 获取所有的value
        values = self.__check_values([data[key] for key in data.keys()], value_list_split)
        # 构造 sql
        pre_sql = f'insert {mode}'
        if mode == 'overwrite':
            pre_sql = 'replace into'
        sql = f'{pre_sql} `{table}` ({keys}) values({",".join(["%s" for i in range(len(data))])})'
        cursor = conn.cursor()
        try:
            cursor.execute(sql, values)
            conn.commit()
        except Exception:
            raise RuntimeError(f"insert fail. run sql: {sql % tuple(values)}")
        cursor.close()

    def insert_dicts(self, table, data, once_length=200, mode='into', values_list_split=','):
        '''
        插入 字典元祖/字典列表 结构数据，形如 ({},{}) 或 [{},{}]，要求每个字典的key全部一致
        :param table: 表名
        :param data: 数据
        :param once_length: 单批插入数据量
        :param mode: 插入模式，into,overwrite,ignore 等，默认into（追加）
        :return:
        '''
        # 基本检测
        self.__check_insert(table, data)
        self.__check_exists_table(table)
        # 获取连接
        conn = self.__conn
        keys = ",".join(map(lambda x: f'`{x}`', data[0].keys()))
        cursor = conn.cursor()
        for i in range(int((len(data) - 1) / once_length) + 1):
            # 分为length批次写入
            length = min(len(data), (i + 1) * once_length)
            pre_sql = f'insert {mode}'
            if mode == 'overwrite':
                pre_sql = f'replace into'
            sql = f'{pre_sql} {table} ({keys}) values '
            values = []
            for dict_obj in data[i * once_length: length]:
                sql += f' ({",".join(["%s" for i in range(len(dict_obj))])}),'
                values += [dict_obj[key] for key in dict_obj.keys()]
            # 剔除末尾空格
            sql = sql[:-1]
            # values 值检测
            values = self.__check_values(values, values_list_split)
            try:
                cursor.execute(sql, values)
            except Exception as e:
                raise RuntimeError(f"insert fail. run sql: {sql % tuple(values)}")
        conn.commit()
        cursor.close()

    def execute(self, sql):
        '''
        Mysql sql执行方法（不建议使用）
        :param sql: 执行sql
        :return:
        '''
        self.__check_conn()
        if BasicCheckUtil.is_none(sql):
            raise ValueError('sql must be not None.')
        conn = self.__conn
        cursor = conn.cursor()
        cursor.execute(sql)
        conn.commit()
        cursor.close()
        return self

    def query(self, sql, args=None, type='dict'):
        '''
        Mysql查询方法
        :param sql: 查询sql，传参使用%s代替
        :param args: 参数对象，元组（tuple）或字典（dict）均可
        :param type: 类型，dict返回形如[{'xxx' : 'xxx'},{'xxx' : 'xxx'}]，tuple返回形如((xxx,xx),(xxx,xx))结果
        :return:
        '''
        self.__check_conn()
        if BasicCheckUtil.is_none(sql):
            raise ValueError('sql must be not None.')
        if type not in ('dict', 'tuple'):
            raise TypeError('type field must be "dict" or "tuple".')

        conn = self.__conn
        if type == 'dict':
            from pymysql.cursors import DictCursor
            cursor = conn.cursor(cursor=DictCursor)
        else:
            cursor = conn.cursor()
        cursor.execute(sql, args=args)
        result = cursor.fetchall()
        conn.commit()
        cursor.close()
        return result

    def insert(self, sql, args=None):
        '''
        todo
        :param sql: sql 字符串
        :param args: 参数元组或列表
        :return:
        '''
        return self.__execute(sql, args, 'insert fail.')

    def update(self, sql, args=None):
        '''
        修改数据
        :param sql: sql 字符串
        :param args: 参数元组或列表
        :return:
        '''
        return self.__execute(sql, args, 'update fail.')

    def delete(self, sql, args=None):
        '''
        删除数据
        :param sql: sql字符串
        :param args: 参数元组或列表
        :return:
        '''
        return self.__execute(sql, args, 'delete fail.')

    def add_column(self, table, name, type):
        '''
        添加一个列
        :param table: 表名
        :param name: 列名
        :param type: python类型
        :return:
        '''
        type = self.__type_to_mysql_type_str(type)
        sql = f'alter table {table} add column {name} {type}'
        self.__execute(sql)

    def close(self):
        '''
        关闭Mysql连接
        :return:
        '''
        self.__conn.close()

    def exists_table(self, table):
        '''
        判断表是否存在
        :param table: 表名
        :return:
        '''
        tables = self.show_tables()
        for tb in tables:
            if table == tb[0]:
                return True
        return False

    def create_table(self, sql, args=None):
        '''
        创建表
        :param sql: sql
        :param args: 参数
        :return:
        '''
        return self.__execute(sql, args)

    def show_tables(self):
        return self.query('show tables', type='tuple')

    def __dict_add_column(self, table, data):
        '''
        字段名， 类型字典
        :param table: 表名
        :param data: {'字段名':类型}
        :return:
        '''
        for k, v in data.items():
            self.add_column(table, k, v)

    def __data_generate_table(self, table, data, split, primary, engine, charset):
        '''
        dict数据生成mysql表
        :param table: 表名
        :param data: dict数据
        :param split:对于tuple，list对象转为str时的分隔符
        :return: None
        '''
        # 存在 id 则删除
        primary_type = 'bigint(19) not null auto_increment'
        if BasicCheckUtil.non_none(data.get(primary)):
            if type(data.get(primary)) == float:
                primary_type = 'decimal(20,6) not null'
            elif type(data.get(primary_type)) != int:
                primary_type = f'varchar({int(len(data.get(primary)) * 1.5)}) not null'
            del data[primary]
        # 将dict数据的value转为数据的类型
        data = self.__dict_to_type(data, split)
        # 构造sql
        sql = f'create table `{table}` (`{primary}` {primary_type},'
        for k, v in data.items():
            if v in (int, bool):
                sql += f'`{k}` bigint(19) default null,'
            elif v == float:
                sql += f'`{k}` decimal(20,6) default null,'
            elif v == str:
                sql += f'`{k}` text default null,'
            else:
                sql += f'`{k}` text default null,'
        sql += f'primary key(`{primary}`))'
        if engine is not None:
            sql += f' engine={engine}'
        if charset is not None:
            sql += f' default charset=\'{charset}\''
        self.__execute(sql)

    def __use_database(self, database):
        '''
        选择默认数据库
        :param database: 数据库名
        :return:
        '''
        self.__execute(f"use {database}")

    def __execute(self, sql, args=None, error_message='run fail.'):
        '''
        Sql 执行方法（核心方法）
        :param sql: sql字符串
        :return:
        '''
        self.__check_conn()
        if BasicCheckUtil.is_empty(sql):
            raise ValueError('sql must be not empty.')
        conn = self.__conn
        cursor = conn.cursor()
        try:
            cursor.execute(sql, args)
            conn.commit()
        except:
            raise RuntimeError(f"{error_message}，fail sql：{sql % (tuple() if args == None else args)}")
        cursor.close()
        return True

    def __dict_to_type(self, data, split):
        '''
        dict结构数据值转为类型
        :param data:
        :return:
        '''
        for key in data.keys():
            data[key] = type(self.__data_to_mysql_data(data[key], split))
        return data

    def __type_to_mysql_type_str(self, type):
        if type in (int, bool):
            return 'bigint(19)'
        elif type in (float,):
            return 'decimal(20,6)'
        elif type in (str,):
            return 'text'
        else:
            return 'text'

    def __data_to_mysql_data(self, data, split):
        '''
        Python 数据类型转 Mysql 数据类型
        :param data:
        :param split:
        :return:
        '''
        if type(data) in (tuple, list):
            return split.join(data)
        if type(data) in (dict,):
            return json.dumps(data, ensure_ascii=False)
        return data

    def __table_column_names(self, table):
        '''
        获取表的所有列
        :param table: 表名
        :return:
        '''
        self.__check_conn()
        conn = self.__conn
        cursor = conn.cursor()
        cursor.execute(f'select * from {table} limit 1')
        columns = [column[0] for column in cursor.description]
        cursor.close()
        return columns

    def __check_values(self, values, split):
        '''
        values 参数检测
        :return:
        '''
        for index in range(len(values)):
            values[index] = self.__data_to_mysql_data(values[index], split)
        return values

    def __check_insert(self, table, data):
        self.__check_conn()
        if BasicCheckUtil.is_empty(table):
            raise ValueError('table name must be not empty')
        if BasicCheckUtil.is_empty(data):
            raise ValueError('data must be not empty')

    def __check_conn(self):
        '''
        检测是否存在Mysql连接
        :return:
        '''
        if BasicCheckUtil.is_none(self.__conn):
            raise ConnectionError('not found useful mysql connect object.')

    def __check_exists_table(self, table):
        if self.exists_table(table) == False:
            raise EOFError(f"not found table name is: {table}")


from ..__init__ import *
def send_message_to_enterprise_wechat(url, message='', message_type='text', mentioned_list=None, mentioned_mobile_list=None):
    '''
    todo 企业微信推送信息函数 (单图片发送功能尚未实现)
    当 message_type = text or markdown, message 值为字符串;
    当 message_type = image; message 值为图片二进制数组
    当 message_type = news, message 值为列表, 列表长度 <= 8, 列表里每个元素如下：
        {
            "title": "这是标题, 必填项, 不超过128字节, 超过会自动截断",
            "description": "这是描述, 非必填, 不超过512个字节, 超过会自动截断",
            "url": "这是点击跳转的链接, 必填",
            "picurl": "这是图片链接, 非必填, 支持JPG、PNG格式, 较好的效果为大图 1068*455, 小图150*150"
        }

    mentioned_list 与 mentioned_mobile_list 仅 message_type = text 时生效
    :param url: 企业微信调用url
    :param message: 推送信息内容
    :param message_type: 推送信息类型,仅限 text文本, markdown, image图片, news图文组合信息
    :param mentioned_list: userid的列表, 提醒群中的指定成员(@某个成员), @all表示提醒所有人, 如果不知道userid可以使用mentioned_mobile_list
    :param mentioned_mobile_list: 手机号列表,提醒手机号对应的群成员(@某个成员), @all表示提醒所有人
    :return: requests.Response 对象
    '''
    import requests
    if type(message) in (dict,):
        res = requests.post(url, json=message)
        return res
    elif type(message_type) in (str, bytearray, bytes, list):
        send_data = {}
        send_data['msgtype'] = message_type
        send_data[message_type] = {}
        if BasicCheckUtil.contains(('text', 'markdown'), message_type):
            send_data[message_type]['content'] = message
        elif BasicCheckUtil.equels('image', message_type):
            import base64
            import hashlib
            send_data[message_type]['base64'] = base64.encodebytes(message).decode('utf-8')
            send_data[message_type]['md5'] = hashlib.md5(message).hexdigest()
            print(send_data[message_type]['base64'])
        elif BasicCheckUtil.equels('news', message_type):
            send_data[message_type]['articles'] = message
        if BasicCheckUtil.non_none(mentioned_list):
            send_data[message_type]['mentioned_list'] = __check_list(data_list=mentioned_list)
        if BasicCheckUtil.non_none(mentioned_mobile_list):
            send_data[message_type]['mentioned_mobile_list'] = __check_list(data_list=mentioned_mobile_list)
        res = requests.post(url, json=send_data)
        return res

def __check_list(data_list):
    if type(data_list) in (str,):
        return [data_list]
    return data_list




from .__init__ import *

class JsonEasyEntry:
    '''
    基于json模型封装实体，适用于规则的多层嵌套json读取
    '''
    __level_prefix = 'level_'
    __init_load_status = False
    __json = None
    __json_text = None
    __struct = None
    __count = None

    def __init__(self, data: str or dict or list or bytes or bytearray):
        import json
        if type(data) in (str,):
            self.__json = json.loads(data)
            self.__json_text = data
        elif type(data) in (dict, list, tuple):
            self.__json = data
            self.__json_text = json.dumps(data)
        self.__struct = {}
        self.__count = {}

    def print(self):
        '''
        输出包含所有结构的json
        :return:
        '''
        import json
        self.__init_load()
        print(json.dumps(self.__struct))

    def get(self, key, level: int=None):
        '''
        获取某个key的结果
        :param key: 字典的key
        :param level: 层级，第一级从0开始，查找指定层级，为None时不限制层级
        :return: list，对应key的结果列表（可能包含多个同名key）
        '''
        self.__init_load()
        if BasicCheckUtil.is_none(level):
            result_list = []
            for level_key, value in self.__struct.items():
                if BasicCheckUtil.non_none(value.get(key)):
                    result_list += value[key]
            return result_list
        level_key = self.__level_prefix + str(level)
        return self.__struct[level_key][key]

    def get_first(self, key, level: int=None):
        '''
        获取某个key的第一个结果
        :param key: 字典的key
        :param level: 层级，第一级从0开始，查找指定层级，为None时不限制层级
        :return: list的第一个元素
        '''
        return self.get_one(key, 0, level)

    def get_last(self, key, level: int=None):
        '''
        获取某个key的最后一个结果
        :param key: 字典的key
        :param level: 层级，第一级从0开始，查找指定层级，为None时不限制层级
        :return: list的最后一个元素
        '''
        if BasicCheckUtil.is_none(level):
            total = 0
            for level_key, value in self.__count.items():
                if BasicCheckUtil.non_none(value.get(key)):
                    total += value[key]
            return self.get_one(key, total - 1, level)
        level_key = self.__level_prefix + str(level)
        return self.get_one(key, self.__count[level_key][key] - 1, level)


    def get_one(self, key, index=0, level: int=None):
        '''
        获取某个key的指定位置结果
        :param key: 字典的key
        :param index: 第 index 次出现
        :param level: 层级，第一级从0开始，查找指定层级，为None时不限制层级
        :return: list 的第 index个元素，从0开始
        '''
        self.__init_load()
        if BasicCheckUtil.is_none(level):
            result_list = []
            for level_key, value in self.__struct.items():
                if BasicCheckUtil.non_none(value.get(key)):
                    result_list += value[key]
                    if len(result_list) > index:
                        return result_list[index]
            return result_list[index]
        level_key = self.__level_prefix + str(level)
        return self.__struct[level_key][key][index]

    def get_original_json(self):
        '''
        获取原始传入dict对象
        :return:dict
        '''
        return self.__json

    def get_original_json_text(self):
        '''
        获取原始传入json字符串
        :return: str
        '''
        return self.__json_text

    def __init_load(self):
        '''
        装载dict对象
        :return:
        '''
        if not self.__init_load_status:
            if self.__json is None:
                return ValueError('当前没有Json对象')
            self.__re_init_load(self.__json)
            self.__init_load_status = True

    def __re_init_load(self, param, level=0):
        '''
        提取嵌套dict至最外层
        :param param:
        :return:
        '''
        level_key = self.__level_prefix + str(level)
        if BasicCheckUtil.is_none(self.__struct.get(level_key)):
            self.__struct[level_key] = {}
            self.__count[level_key] = {}
        if type(param) in (dict,):
            for key, value in param.items():
                count = self.__count[level_key].get(key)
                if count is None:
                    count = 0
                    self.__struct[level_key][key] = []
                self.__struct[level_key][key].insert(count, value)
                count += 1
                self.__count[level_key][key] = count
                self.__re_init_load(value, level + 1)
        elif type(param) in (list, tuple):
            for single in param:
                self.__re_init_load(single, level + 1)
